"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathResolver = void 0;
const path_to_regexp_1 = require("path-to-regexp");
const check_1 = require("./check");
const clone_1 = require("./clone");
const init_1 = require("./init");
class PathResolver {
    opt;
    routes = [];
    constructor(routes, opt = { isDebug: false }) {
        this.opt = opt;
        this.routes = routes.map(route => init_1.Init.route(route, '/'));
    }
    resolve(pathname) {
        this.log(`resolving '${pathname}'`);
        return this.find(pathname, this.routes);
    }
    find(pathname, routes, parentRoute) {
        if (!routes)
            return;
        for (let i = 0; i < routes.length; i++) {
            let route = routes[i];
            if (check_1.Check.skipBranch(route.path, pathname)) {
                this.log(`[x] ${route.path}, skip branch`);
                continue;
            }
            const match = (0, path_to_regexp_1.match)(route.path)(pathname);
            this.log(`[${match ? 'v' : 'x'}] ${route.path}`);
            if (match && !check_1.Check.needToMatchChildren(route)) {
                route = clone_1.Clone.route(route);
                const urlParams = match.params;
                if (route.redirectTo) {
                    route.redirectTo = (0, path_to_regexp_1.compile)(route.redirectTo)(urlParams);
                }
                if (route.customTo?.pathname) {
                    route.customTo.pathname = (0, path_to_regexp_1.compile)(route.customTo.pathname)(urlParams);
                }
                return { route, urlParams, parentRoute };
            }
            const found = this.find(pathname, route.children, route);
            if (found)
                return found;
        }
    }
    correctResultFromAction(pathname, result, route, parentRoute) {
        if (result.redirectTo === undefined && result.customTo === undefined)
            return;
        const parentPath = parentRoute ? parentRoute.path : '/';
        result.redirectTo = init_1.Init.to(result.redirectTo, parentPath);
        result.customTo = init_1.Init.customTo(result.customTo, parentPath);
        const match = (0, path_to_regexp_1.match)(route.path)(pathname);
        if (match) {
            const urlParams = match.params;
            if (result.redirectTo !== undefined) {
                result.redirectTo = (0, path_to_regexp_1.compile)(result.redirectTo)(urlParams);
            }
            if (result.customTo?.pathname) {
                result.customTo.pathname = (0, path_to_regexp_1.compile)(result.customTo.pathname)(urlParams);
            }
        }
    }
    log(path) {
        if (this.opt.isDebug)
            console.log(' ', path);
    }
}
exports.PathResolver = PathResolver;
